import java.util.*;
import java.io.*;

/**
 * Regular adaptive arithmetic coding model.
 */
public class AdaptiveModel implements ArithmeticModel
{
	int[] counts = MAMUtil.setDefaultDistribution();
	byte[] chars = MAMUtil.setChars();
	int currentIndex = -1;
	boolean indexOk = false;

	public AdaptiveModel()
	{
		// nothing
	}

	public long lowInterval(int symbol)
	{
		if (!indexOk)
		{
			currentIndex = symbolToIndex(symbol);
			indexOk = true;
		}

		return previousCount();
	}

	/** Gets cumulative symbol count of the sybol with the previous index
	(probabitily other than 0 or for the first symbol) in encoding. */
	private long previousCount()
	{
		int temp = currentIndex;
		while (true)
		{
			if (temp <= 0)
				return (long)temp;
			if (counts[temp - 1] != 0)
				return (long)counts[temp - 1];
			temp--;
		}
	}

	public long highInterval(int symbol)
	{
		if (!indexOk)
		{
			currentIndex = symbolToIndex(symbol);
			indexOk = true;
		}
		
		return (long)counts[currentIndex];
	}

	public long totalCount()
	{
		return (long)counts[256];
	}

	public void updateModel(int symbol)
	{
		while (currentIndex < counts.length)
		{
			counts[currentIndex]++;
			currentIndex++;
		}
		
		if (counts[255] >= Integer.MAX_VALUE - 2)
			rescaleCounts();
		
		indexOk = false;
	}

	private void rescaleCounts()
	{
		int[] freqs = new int[counts.length];
		for (int i = 1; i < freqs.length; ++i)
			freqs[i] = (counts[i] - counts[i-1] + 1) / 2;
		
		counts[0] = (counts[0] + 1) / 2;
		for (int i = 1; i < counts.length; ++i)
			counts[i] = counts[i-1] + freqs[i];
	}
	
	private int symbolToIndex(int symbol)
	{
		return (symbol == -1) ? 256 : Arrays.binarySearch(chars, (byte)symbol);
	}

	public int countToSymbol(int cum)
	{
		int index = 0;
		while (counts[index] <= cum)
			index++;

		currentIndex = index;
		indexOk      = true;

		if (index == 256)
			return -129; // -1 -128
		return (int)chars[index];
       }

       public void reset() {
         counts       = MAMUtil.setDefaultDistribution();
         chars        = MAMUtil.setChars();
         currentIndex = -1;
         indexOk      = false;
       }

}
