/**
 * Common interface for different kinds of models that can be used
 * together with the arithmetic coding.
 */
public interface ArithmeticModel {

	/**
	* Informs the model with last seen symbol.
	* Model and its frequency counts are updated accordingly
	*/
	public void updateModel(int symbol);

	/**
	* Returns the total count.
	*/
	public long totalCount();

	/**
	* Returns the low interval cumulative count of a symbol
	*/
	public long lowInterval(int symbol);
	
	/**
	* Returns the high interval cumulative count of a symbol
	*/
	public long highInterval(int symbol);

	/**
	* Return the symbol with the cumulative count
	*/
	public int countToSymbol(int cum);

        /** State reset to the initial values **/
        public void reset();

}
