import java.io.*;
import java.util.*;

/**
 * Burrows-Wheeler transformation decoding codec.
 */
public class BWTDecode implements Codec {

  public void processTransformation(InputStream  in,
                                    OutputStream out) throws IOException {
      final int offset = Byte.MIN_VALUE;
      int       bSize  = MAMUtil.readInt(in);
      byte[]    block  = new byte[bSize+4]; // Add start index
      byte[]    buffer = new byte[block.length];
      int[]     indi   = null;
      int[]     bucs   = new int[256];
      int       avail;
      int       k;
      
      while ((avail=MAMUtil.read(in, block)) > 0) {
          avail -= 4; // Cut off the start index

          // Initialization of the indices
          if ((indi == null) || (indi.length > avail)) {
             indi = new int[avail];
          }

          Arrays.fill(bucs, 0);
          for (int i=0; i<avail; i++) {
              bucs[block[i]-offset]++;
          }
          k = 0;
          for (int i=0; i<bucs.length; i++) {
              for (int j=0; j<bucs[i]; j++) {
                  buffer[k++] = (byte)(i+offset);
	      }
	  }
          assert(k==buffer.length);
          k = 0;
          for (int i=0; i<bucs.length; i++) {
              while (i+offset>buffer[k] && k<avail) {
		  k++;
	      }
              bucs[i] = k;
	  }
          for (int i=0; i<avail; i++) {
              indi[bucs[block[i]-offset]++] = i;
          }
          k = MAMUtil.toInt(block, avail);
          for (int i=0; i<avail; i++) {
	      out.write(block[k]);
              k = indi[k];
	  }
          out.flush();
      }
  }

}
