import java.io.*;
import java.util.*;

/**
 * Burrows-Wheeler transformation encoding codec.
 */
public class BWTEncode implements Codec {

  /** Default length for the encoding window **/
  static public final int DEFAULT_BLOCK_SIZE = 500*1024;

  private int blockSize = DEFAULT_BLOCK_SIZE;

  public void processTransformation(InputStream  in,
                                    OutputStream out) throws IOException {
    byte[] block = new byte[blockSize];
    int[]  indi  = null;
    int    avail;
    int    start;

    out.write(MAMUtil.toBytes(block.length));
    while ((avail=MAMUtil.read(in, block)) > 0) {
      // Initialization of the indices
      if ((indi == null) || (indi.length > avail)) {
         indi = new int[avail];
      }
      for (int i=0; i<indi.length; i++) {
        indi[i] = i;
      }

      // Sorting
      cacheC = new int[3][3];
      sort(0, indi.length-1, indi, block);

      // Printing the results
      start = -1;
      for (int i=0; i<avail; i++) {
        out.write(block[(indi[i]+avail-1)%avail]);
        if (indi[i] == 1) {
           start = i;
        }
      }
      out.write(MAMUtil.toBytes(start));
      out.flush();
    }
  }

  /**
   * Merge sort for the given array.
   */
  void sort(int    s,     // start index
            int    e,     // end index
            int[]  array,
            byte[] data) {
      if (s >= e) return; // Done!

      int mid = (s+e) /2;

      // Sort both halves...
      sort(s,     mid, array, data);
      sort(mid+1, e,   array, data);

      // Merge sorted regions...
      int sE = mid;   // End of the first region
      int eS = mid+1; // Start of the second region
      if ((data!=null) && compare(array[sE], array[eS], array.length, data) == 0)
         return;
      while ((s<=sE) && (e >= eS)) {
	  if (((data == null) && (array[s] < array[eS]))||
	      ((data != null) && compare(array[s],array[eS],array.length,data)<0)) {
	      s++;
	  } else {
	      int t = array[eS];
              for (int i=eS-1; i>=s; i--) {
                  array[i+1] = array[i];
	      }
              array[s++] = t;
              sE++;
              eS++;
	  }
      }
  }

  int[][] cacheC;
  int     phase = 0;

  private int compare(int    a,
                      int    b,
                      int    size,
                      byte[] data) {
      int s = size;
      if (a == b)
          return 0;
      for (int j=0; j<cacheC.length; j++) {
          if (cacheC[j][0] == a &&
              cacheC[j][1] == b)
             return cacheC[j][2];
      }
      while (data[a] == data[b]) {
	  if (++a == size)
	      a = 0;
          if (++b == size)
	      b = 0;
          if (--s <= 0)
	      return 0;
      }
      int d = (data[a] - data[b]);
      if (s<size-100) {
         if (++phase > cacheC.length-1) phase = 0;
         cacheC[phase][0] = a;
         cacheC[phase][1] = b;
         cacheC[phase][2] = d;
      }
      return d;
  }

}
