import java.util.*;
import junit.framework.*;

/**
 * Unit tests for Burrows-Wheeler transformation.
 */
public class BWTTest extends TestCase {

  static private final String E1_ORIGINAL =
      "SIX.MIXED.PIXIES.SIFT.SIXTY.PIXIE.DUST.BOXES";
  static private final String E1_ENCODED =
      new String(MAMUtil.toBytes(BWTEncode.DEFAULT_BLOCK_SIZE))+
      "TEXYDST.E.IXIXIXXSSMPPS.B..E.S.EUSFXDIIOIIIT"+
      new String(MAMUtil.toBytes(18));

  public void testEncoding() {
    String ret = MAMUtil.callCodec(new BWTEncode(), E1_ORIGINAL);

    assertEquals(E1_ORIGINAL.length()+8, ret.length());
    assertEquals(E1_ENCODED, ret);
  }

  public void testDecoding() {
      String ret = MAMUtil.callCodec(new BWTDecode(), E1_ENCODED);

      assertEquals(E1_ENCODED.length()-8, ret.length());
      assertEquals(E1_ORIGINAL, ret);
  }

  public void testSingleCharSeqeuence() {
      long   st  = System.currentTimeMillis();
      String ret = MAMUtil.callCodec(new BWTEncode(), new String(new byte[10000]));
      long   et  = System.currentTimeMillis();

      assertTrue("Conversion took "+((et-st)/1000.0)+"s", et-st < 5*1000);
  }

  public void testBinaryContent() {
    byte[] cnt = new byte[256];
    int    j   = 0;
    String val;
    String enc;

    for (int i=Byte.MIN_VALUE; i<=Byte.MAX_VALUE; i++) {
       cnt[j++] = (byte)i;
    }
    val = new String(cnt);
    enc = MAMUtil.callCodec(new BWTEncode(), val);
    enc = MAMUtil.callCodec(new BWTDecode(), enc);
    assertEquals(val, enc);
  }

  public void testSort() {
      int[] a = new int[] { 1, 2, 3, 4, 5 };
      int[] b = new int[] { 8, 4, 6, 1, 9 };
      int[] c = new int[] { 4, 3, 2, 1 };
      int[] d = new int[] { 9, 3, 2, 1, 0 };

      BWTEncode inst = new BWTEncode();

      inst.sort(0, a.length-1, a, null);
      inst.sort(0, b.length-1, b, null);
      inst.sort(0, c.length-1, c, null);
      inst.sort(1, 3,          d, null);
      assertEquals("[1, 2, 3, 4, 5]", Arrays.toString(a));
      assertEquals("[1, 4, 6, 8, 9]", Arrays.toString(b));
      assertEquals("[1, 2, 3, 4]",    Arrays.toString(c));
      assertEquals("[9, 1, 2, 3, 0]", Arrays.toString(d));
  }

}
