import java.io.*;

/**
 * This output stream provides an eight bit long buffer
 * for writing individual bits.
 */
public class BinaryInputStream extends InputStream {

  private InputStream in;

  private byte buffer  = 0;
  private int  pointer = 8;

  /**
   * Create a binary stream on the top of the given stream.
   */
  public BinaryInputStream(InputStream stream) {
    in = stream;
  }

  public int available() throws IOException {
    return in.available();
  }

  public int nextBit() throws IOException {
	if (pointer == 8)
	{
		int bytevalue = this.read();
	
		if (bytevalue == -1)
			throw new EOFException();

		buffer = (byte)bytevalue;
		pointer = 0;
	}

	int result = (buffer >>> 7 - pointer) & 0x1; // works
	pointer++;

	return result;
  }

  public int read() throws IOException {
	return in.read();
  }

  public int readBits(int count) throws IOException {
	int value = this.nextBit(); // 0'th bit
	int bit;
	for (int i = 1; i < count; i++)
	{
		try {
			bit = this.nextBit();
		} catch (EOFException e) {

			value <<= count - i;
			return value;
		}
		value <<= 1;
		value += bit;
	}

	return value;
	}

}
