import java.io.*;

/**
 * This output stream provides an eight bit long buffer
 * for writing individual bits.
 */
public class BinaryOutputStream extends OutputStream {

  private OutputStream out;

  private byte buffer   = 0;
  private int  bCounter = 0;

  /**
   * Create a binary stream on the top of the given stream.
   */
  public BinaryOutputStream(OutputStream stream) {
    out = stream;
  }

  public void write(int b) throws IOException {
    if (bCounter > 0) {
       flushBitBuffer();
    }
    out.write(b);
  }

  /**
   * Flush the bit buffer by padding it with zeros.
   */
  public void flushBitBuffer() throws IOException {
    while (bCounter != 0) {
      writeBit(false);
    }
    out.flush();
  }

  public boolean writeBit(boolean bit) throws IOException {
    boolean ret = false;
    buffer = (byte)(buffer << 1);
    if (bit) {
       buffer = (byte)(buffer | 0x00000001);
    }
    bCounter++;
    if (bCounter > 7) {
       ret      = true;
       bCounter = 0;
       out.write(buffer);
    }
    return ret;
  }

}
