import java.io.*;
import java.util.*;

/**
 * Decoding for the artimetic coding.
 */
public class MAMDecode implements Codec {

  private ArithmeticModel model;

  public MAMDecode() {
    this(new AdaptiveModel());
  }

  public MAMDecode(ArithmeticModel mod) {
    model = mod;
  }

  public void setModel(ArithmeticModel mod) {
    model = mod;
  }

  public synchronized void processTransformation(InputStream  in,
                                                 OutputStream o) throws IOException {
    model.reset();

    BinaryInputStream  bIn = new BinaryInputStream(in);
    BinaryOutputStream out = new BinaryOutputStream(o);

    int bits              = 31;                    // bits used for integer arithmetic
    int maxValue          = Integer.MAX_VALUE;     // max value of bit buffer
    int firstQuarterValue = (maxValue / 4) + 1;    // first quarter value
    int halfValue         = 2 * firstQuarterValue; // half point value for buffer
    int thirdQuarterValue = 3 * firstQuarterValue; // third quarter value

    int low   = 0;
    int high  = maxValue;
    int value = bIn.readBits(bits);                // main bit buffer for value (31 bits)
    long range;
    // main loop, decodes one symbol per loop round
    while (true) {
       int cum;

       range = ((long)high - (long)low) + 1;
       cum   = (int)(((((long)((long)value-(long)low)+1) * model.totalCount())-1) / range); 

       int symbol = model.countToSymbol(cum); // <new/>
       high = low + (int)((range * model.highInterval(symbol)) / model.totalCount()) - 1; 
       low  = low + (int)((range * model.lowInterval(symbol))  / model.totalCount());

       if (symbol == -129) // -1 - 128
          break;
       out.write((byte)symbol);

       // rescale intervals
       while (true) {
          if (high < halfValue) {
             // haha, do nothing!
             // falls through and doubles low, high and value
          }
          else if (low >= halfValue) {
             value -= halfValue;
             low   -= halfValue;
             high  -= halfValue;
          }
          else if (low >= firstQuarterValue && high < thirdQuarterValue) {
             value -= firstQuarterValue;
             low   -= firstQuarterValue;
             high  -= firstQuarterValue;
          }
          else {
             break;
          }
          low  = 2 * low;
          high = (2 * high) + 1;
          try {
             value = (2 * value) + bIn.nextBit();
          } catch (IOException e) {
             value = (2 * value);
          }
       }
       model.updateModel(symbol);
    }
    out.flushBitBuffer();
    out.flush();
  }

  public static void main(String[] argv) throws IOException {
    Codec bwt = new BWTDecode();
    Codec m2f = new M2FDecode();
    Codec mam = new MAMDecode();

    PipedOutputStream outMAM = new PipedOutputStream();
    PipedInputStream  inM2F  = new PipedInputStream(outMAM);
    PipedOutputStream outM2F = new PipedOutputStream();
    PipedInputStream  inBWT  = new PipedInputStream(outM2F);

    //MAMUtil.callCodec(mam, System.in, System.out); // By pass the pre-processing
    MAMUtil.callCodec(mam, System.in, outMAM);
    MAMUtil.callCodec(m2f, inM2F,     outM2F);
    MAMUtil.callCodec(bwt, inBWT,     System.out);
  }

}
