import java.util.*;
import junit.framework.*;

public class MAMTest extends TestCase {

  public static void testCoding() {
      String original;
      String encoded;
      MAMEncode enc = new MAMEncode();
      MAMDecode dec = new MAMDecode();

      StringBuffer buf = new StringBuffer(50000);
      for (int i=0; i<5000; i++) {
         buf.append("0123456789");
      }
      original = buf.toString();

      // Adaptive model
      encoded  = MAMUtil.callCodec(enc, original);
      assertTrue(original.length() > encoded.length() * 2.3);
      assertEquals(original, MAMUtil.callCodec(dec, encoded));

      // PPM
      enc.setModel(new PPMModel());
      dec.setModel(new PPMModel());
      encoded  = MAMUtil.callCodec(enc, original);
      assertTrue(original.length() > encoded.length() * 2.3);
      assertEquals(original, MAMUtil.callCodec(dec, encoded));
  }

  public void testCompressionRate() {
    MAMEncode codec = new MAMEncode();
    String    input = new String(new byte[100000]);
    String    res;

    // Adaptive model
    res = MAMUtil.callCodec(codec, input);
    assertTrue("Compressing "+input.length()+" bytes into "+res.length()+" bytes.",
               res.length() * 300 < input.length());

    // PPM model
    codec.setModel(new PPMModel());
    res = MAMUtil.callCodec(codec, input);
    assertTrue("Compressing "+input.length()+" bytes into "+res.length()+" bytes.",
               res.length() * 100 < input.length());
  }

  public void testByteEncodeDecodeCycle() {
    final String I1  = "Each byte represents a symbol!";
    MAMEncode    enc = new MAMEncode();
    MAMDecode    dec = new MAMDecode();
    String       res;

    // Adaptive model
    res = MAMUtil.callCodec(enc, I1);  // Encode
    res = MAMUtil.callCodec(dec, res); // Decode back to original form
    assertEquals(I1.length(), res.length());
    assertEquals(I1,          res);

    // PPM model
    enc.setModel(new PPMModel());
    dec.setModel(new PPMModel());
    res = MAMUtil.callCodec(enc, I1);  // Encode
    res = MAMUtil.callCodec(dec, res); // Decode back to original form
    assertEquals(I1.length(), res.length());
    assertEquals(I1,          res);
  }

  public void testBinaryContent() {
    byte[] cnt = new byte[257];
    int    j   = 0;
    String val;
    String enc;

    for (int i=Byte.MIN_VALUE; i<Byte.MAX_VALUE; i++) {
       cnt[j++] = (byte)i;
    }
    cnt[256] = Byte.MIN_VALUE;
    val      = new String(cnt);

    // Adaptive model
    enc = MAMUtil.callCodec(new MAMEncode(), val);
    enc = MAMUtil.callCodec(new MAMDecode(), enc);
    assertEquals(val, enc);

    // PPM model
    enc = MAMUtil.callCodec(new MAMEncode(new PPMModel()), val);
    enc = MAMUtil.callCodec(new MAMDecode(new PPMModel()), enc);
    assertEquals(val, enc);
  }

}
